import sys, os
from astropy import wcs
from astropy.io import fits
import warnings
from astropy.utils.exceptions import AstropyWarning
from astropy.coordinates import SkyCoord
from astropy import units as u
from astroquery.astrometry_net import AstrometryNet
import sep
from astropy.table import Table, hstack, Column
import subprocess

import files 

class Astrometry_NET():
    def __init__(self):
        self.a=1
        
    def Local_Photometry(self, name):
        src=Table()
        hdu=fits.open(name)
        data=hdu[0].data
        data = data.byteswap(inplace=True).newbyteorder()
        data=data.astype('float64')
        bkg=sep.Background(data, bw=64, bh=64, fw=5, fh=5)      
        sources=sep.extract(data-bkg,5.0, minarea=25)
        src['X']=sources['x']
        src['Y']=sources['y']
        src['FLUX']=sources['flux']
        src['FLAG']=sources['flag']
        src=src[src['FLAG']==0]
        src=src[src['FLUX']>15000.]
        src.sort('FLUX')
        src.reverse()
         
        return src
        
        return True
    def SIMBAD_CENTER(self, name):
		# find center of object from simbad
        c = SkyCoord.from_name(name)  		
        return c.ra, c.dec
    
    def Solution_NET(self, image, api_key='', ra=0, dec=0):
       
        ast=AstrometryNet()
        ast.api_key=api_key
        src=self.Local_Photometry(image['WORKING_NAME'])
        
        print(src)
        
        #sys.exit()
        #ra=ra*u.deg/(1.*u.deg)
        #wcs_header = ast.solve_from_image(image['WORKING_NAME'], force_image_upload=True)
        wcs_header = ast.solve_from_source_list(src['X'], src['Y'],
                                                image['NAXIS1'], image['NAXIS2'], 
                                                downsample_factor=1, crpix_center=True,
                                                scale_type='ul',scale_lower=0.5,
                                                scale_upper=0.6, parity=2, center_ra=float(ra.value),
                                                center_dec=float(dec.value), radius=1, scale_units='degwidth' )
                                                #center_ra=ra.value, center_dec=dec.value)
                                                
                                               # , parity=0, )
        
        #                                        radius=5, center_ra=ra.value, center_dec=dec.value,
         #                                        )
#-p --overwrite --downsample 2 --cpulimit 5 --guess-scale --scale-units degwidth --scale-low '+ str(min(fov_x, fov_y)) + ' --scale-high '+str(max(fov_x, fov_y))+'  --ra '+str(ra)+ ' --dec ' + str(dec) +' --radius '+str(radius)+ ' --tweak-order '+ str(degr)+' '+ file_name + ' > '+self.tmp_dir+'ll'
		#print(solve)
        #print(ast.api_key)
        #wcs_header = ast.solve_from_image(image_name)
        print(wcs_header)
        return True
    
    def Solution_Local(self,image, ra=0, dec=0,radius=0.8, fov_x=0.5,fov_y=0.5, tmp_dir=''):       
        degr=2
        #Stmp_name=files.Copy_To_Tmp(image['ORIGINAL_NAME'],tmp_dir)
        tmp_name='/home/parimucha/photometry/tmp/aa-070V.fit'
        #solve = 'solve-field -p --overwrite --downsample 2 --cpulimit 5 --guess-scale --scale-units degwidth --scale-low '+ str(min(fov_x, fov_y)) + ' --scale-high '+str(max(fov_x, fov_y))+'  --ra '+str(ra.value)+ ' --dec ' + str(dec.value) +' --radius '+str(radius)+ ' --tweak-order '+ str(degr)+' '+ tmp_name + ' > '+tmp_dir+'ll'
        solve = 'solve-field -p --overwrite -D '+tmp_dir + ' '+ image['WORKING_NAME']
        
        
        print()
        print(solve)
        #os.system(solve)
		

        subprocess.run(["solve-field", "--overwrite","/home/parimucha/2018-08-17/tmp/aa-070V.xy" ], shell=False)
        
        return True
    
    
class Astrometry():
	#def __init__(self, file_name, global_parameters, ra, dec, radius, degr=3, fov_x=5, fov_y=3):	
	def __init__(self, tmp_dir, global_parameters):		
		#self.file_name = file_name
		#self.ra=ra
		#self.dec=dec
		#self.radius=radius
		#self.degr=degr
		#self.fov_x=fov_x
		#self.fov_y=fov_y
		self.ASTROMETRY=global_parameters['ASTROMETRY']
		self.tmp_dir=tmp_dir
	
	def Clear(self):
		#print (self.file_name)
		return True
	
	def Simbad_Center(self, name):
		# find center of object from simbad
		c = SkyCoord.from_name(name)  		
		return c.ra, c.dec
		
	def Solve(self, file_name, ra, dec, radius, degr=3, fov_x=5, fov_y=3):
		#print(fov_x, fov_y)
		solve = self.ASTROMETRY+' -p --overwrite --downsample 2 --cpulimit 5 --guess-scale --scale-units degwidth --scale-low '+ str(min(fov_x, fov_y)) + ' --scale-high '+str(max(fov_x, fov_y))+'  --ra '+str(ra)+ ' --dec ' + str(dec) +' --radius '+str(radius)+ ' --tweak-order '+ str(degr)+' '+ file_name + ' > '+self.tmp_dir+'ll'
		#print(solve)
		os.system(solve)
		#sys.exit()
		name_path=os.path.dirname(file_name) +'/' 
		#print()
		solved_file=name_path+ os.path.basename(file_name.strip()).replace('.fit', '.solved')
		if os.path.isfile(solved_file):
			solved=True
			wcs_header=name_path+ os.path.basename(file_name.strip()).replace('.fit', '.wcs')
			#name_path+os.path.basename(self.file_name.strip()).split('.')[0]+'.wcs'
			wcs_new=name_path+ os.path.basename(file_name.strip()).replace('.fit', '.new')
			#name_path+os.path.basename(self.file_name.strip()).split('.')[0]+'.new'			
			
		else:
			solved=False
			wcs_header=''
			wcs_new=''
		#Astrometry.Clear(self)
		return solved, wcs_header, wcs_new
	
	
def FOV(filename=None, header=None):
	if header is None:
		filename=filename
		hdu=fits.open(filename) 
		header=hdu[0].header
	else:
		header=header
	warnings.filterwarnings('ignore', category=AstropyWarning)	
	w = wcs.WCS(header)
	
	scale=wcs.utils.proj_plane_pixel_scales(w)
	pixel_scale=scale[0]
	
	fov1=header['NAXIS1']*scale[0]
	fov2=header['NAXIS2']*scale[1]
			
	return fov1, fov2, pixel_scale

def CENTRAL(filename=None, header=None):
	if header is None:
		filename=filename
		hdu=fits.open(filename) 
		header=hdu[0].header
	else:
		header=header
		
	warnings.filterwarnings('ignore', category=AstropyWarning)	
	w = wcs.WCS(header)
	
	centralX, centralY=w.all_pix2world(int(header['NAXIS1']/2), int(header['NAXIS2']/2), 0, ra_dec_order=True)
	
			
	return centralX, centralY
