import sys, os, shutil
import numpy as np
import numpy.ma as ma
from astropy.table import Table, Column, hstack, vstack
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
#from scipy.odr import RealData,Model,ODR
from astropy.modeling import models, fitting
from astropy.stats import sigma_clip


class Calibration():
    def __init__(self, filter_list=None, catalog_path=None, tmp_dir=None):	
        self.a=1
	
    def lin_model(param, x):
        param[0]+param[1]*x
        
    def Calibrate(self, table, filter='V', sigma=3.0):
        
        model=models.Linear1D() 
        fitter = fitting.LevMarLSQFitter()        
        sigma_fit=fitting.FittingWithOutlierRemoval(fitting.LevMarLSQFitter(), sigma_clip, niter=3, sigma=sigma)
        
        #remove variable from calibration
        idx=ma.where(~ma.isin(table['V'],0))
        tmp_table=table[idx]
        
        
        mag=tmp_table['MAG_AUTO']
        mag_err=tmp_table['MAGERR_AUTO']
        
        
        if filter=='V' or filter=='Vb':
            catalog_mag=tmp_table['Vmag']
            catalog_mag_err=tmp_table['e_Vmag']
        if filter=='B' or filter=='Bb':
            catalog_mag=tmp_table['Bmag']
            catalog_mag_err=tmp_table['e_Bmag']
        if filter=='R' or filter=='Rb' or filter=='Rc':            
            catalog_mag=tmp_table['r_mag']-0.2936*(tmp_table['r_mag']-tmp_table['i_mag']) - 0.1439
            catalog_mag_err=tmp_table['e_i_mag']+tmp_table['e_r_mag']
        if filter=='I' or filter=='Ib' or filter=='Ic':            
            catalog_mag=tmp_table['r_mag']-1.2444*(tmp_table['r_mag']-tmp_table['i_mag']) - 0.3820
            catalog_mag_err=tmp_table['e_i_mag']+tmp_table['e_r_mag']
        if filter=='g':
            catalog_mag=tmp_table['g_mag']
            catalog_mag_err=tmp_table['e_g_mag']
        if filter=='r':
            catalog_mag=tmp_table['r_mag']
            catalog_mag_err=tmp_table['e_r_mag']
        if filter=='i':
            catalog_mag=tmp_table['i_mag']
            catalog_mag_err=tmp_table['e_i_mag']      
              
        sigma_fitted_model, mask=sigma_fit(model, catalog_mag, mag, weights=1./mag_err)
        sigma_filters_catalog_mag=ma.masked_array(catalog_mag,mask=mask)
        sigma_filtered_data=ma.masked_array(mag,mask=mask)
        sigma_filtered_data_error=ma.masked_array(mag_err,mask=mask)

        ff,cov=np.ma.polyfit(sigma_filters_catalog_mag,sigma_filtered_data,1,w=1./sigma_filtered_data_error,full=False, cov=True)
        
        slope_error=np.sqrt(cov[0,0])
        intercept_error=np.sqrt(cov[1,1])
        zero_point=ff[1]/ff[0]
        zero_point_error=np.sqrt((intercept_error/ff[1])**2+ (slope_error/ff[0]))
        table['CALIBRATED']=(table['MAG_AUTO']-ff[1])/ff[0]
        table['CALIBRATED_ERR']=np.sqrt(table['MAGERR_AUTO']**2+zero_point_error**2)
        number=len(tmp_table)
      
        
        return table, zero_point, zero_point_error, number
             

 
