import sys, os, shutil
from astropy.coordinates.sky_coordinate import SkyCoord, Angle
from astropy import units as u
from astropy.io import ascii
from astropy.table import Table, Column, hstack, vstack, unique
import numpy as np
from collections import Counter
from copy import deepcopy

from astroquery.simbad import Simbad
from astroquery.vizier import Vizier

from matplotlib import pyplot as plt



# load_catalog_urat
# load_catalog_usnob
# load_SIMBAD

# find_on_disk
# load_catalog_file


class Catalogs():
    def __init__(self, catalog_type=None, catalog_path=None, tmp_dir=None):	
        Vizier.ROW_LIMIT=-1
        Simbad.ROW_LIMIT=-1

    def Find_Local_Catalog(self, coord, path):
        files=os.listdir(path)
        table=[]
        name=''
        for f in files:
           
            tab=ascii.read(path+f)
            name=path+f
            ra=tab.meta['comments'][0]
            dec=tab.meta['comments'][1]            
            c=SkyCoord(ra, dec ,unit=(u.deg, u.deg), frame='icrs')
           # print(coord.separation(c))
            #sys.exit()
            if coord.separation(c)<=0.08*u.deg:
                #tables[tab.meta['comments'][0]]=tab
                table=tab
        return table, name
    
    
    def Load_Vizier(self, coord, catalog='USNOB', fov=0.6):
        #catalogs USNOB, VSX, APASS, sdss12
        
        v=Vizier(column_filters={"B1mag":"<18"})
        v.ROW_LIMIT=-1
		
        result = v.query_region(coord, width=fov*u.deg, catalog=[catalog])
        if catalog=='USNOB':
            cat=result[0]['USNO-B1.0','RAJ2000', 'DEJ2000']
            cat.meta['comments']=['USNOB',str(round(coord.ra.degree,5)), str(round(coord.dec.degree,5)), str(fov)]
        if catalog=='VSX':
            cat=result[0]['Name','RAJ2000', 'DEJ2000','V','Type', 'Period']
            cat.meta['comments']=['VSX',str(round(coord.ra.degree,5)), str(round(coord.dec.degree,5)), str(fov)]
        if catalog=='APASS':
            cat=result[0]["recno" ,"RAJ2000", "DEJ2000", "Vmag","e_Vmag","Bmag","e_Bmag","g_mag", "e_g_mag","r_mag", "e_r_mag","i_mag", "e_i_mag" ]
            cat.meta['comments']=['APASS',str(round(coord.ra.degree,5)), str(round(coord.dec.degree,5)), str(fov)]
        #if catalog=='SDSS12':
        #    cat=result[0]["recno" ,"RAJ2000", "DEJ2000", "Vmag","e_Vmag","Bmag","e_Bmag","g_mag", "e_g_mag","r_mag", "e_r_mag","i_mag", "e_i_mag" ]
        #    cat.meta['comments']=['SDSS12',str(round(coord.ra.degree,5)), str(round(coord.dec.degree,5)), str(fov)]
        return cat
    
    
    def Match_All(self, catalog_usnob,catalog_apass, catalog_vsx):
        
        if len(catalog_usnob)>=len(catalog_apass):
            c_usnob=SkyCoord(catalog_usnob['RAJ2000'],catalog_usnob['DEJ2000'],frame='icrs',unit='deg')             
            c_apass=SkyCoord(catalog_apass['RAJ2000'],catalog_apass['DEJ2000'], frame='icrs', unit='deg')
            idx, d2d, d3d = c_apass.match_to_catalog_sky(c_usnob,nthneighbor=1)
            MatchedTable=catalog_usnob[idx]
            stacked=hstack([MatchedTable,catalog_apass])
            tolerance=d2d<0.0001*u.deg
           
            
            stacked.rename_column('RAJ2000_1', 'RAJ2000')
            stacked.rename_column('DEJ2000_1', 'DEJ2000')
            stacked.remove_column('RAJ2000_2')   
            stacked.remove_column('DEJ2000_2')                   
            
            stacked1=vstack([stacked,catalog_usnob])
            by_name=stacked1.group_by(['RAJ2000'])
           
            catalog_un=unique(by_name,keys=['RAJ2000'], keep='first')
            
            #print(un)
            c_un=SkyCoord(catalog_un['RAJ2000'],catalog_un['DEJ2000'],frame='icrs',unit='deg')             
            c_vsx=SkyCoord(catalog_vsx['RAJ2000'],catalog_vsx['DEJ2000'],frame='icrs',unit='deg')
            idx, d2d, d3d = c_vsx.match_to_catalog_sky(c_un,nthneighbor=1)            
            MatchedTable=catalog_un[idx]
            
            stacked=hstack([MatchedTable,catalog_vsx])
            tolerance=d2d<0.0001*u.deg
            
            stacked.rename_column('RAJ2000_1', 'RAJ2000')
            stacked.rename_column('DEJ2000_1', 'DEJ2000')
            stacked.remove_column('RAJ2000_2')   
            stacked.remove_column('DEJ2000_2')  
            stacked1=vstack([stacked,catalog_un])
            by_name=stacked1.group_by(['RAJ2000'])
            catalog=unique(by_name,keys=['RAJ2000'], keep='first')
            
           # catalog.meta={}
            
            
        return catalog
    
    
    def Match(self,measurements, catalog):
        
        c_measurements=SkyCoord(measurements['ALPHA'],measurements['DELTA'], frame='icrs', unit='deg')
        c_catalog=SkyCoord(catalog['RAJ2000'],catalog['DEJ2000'],frame='icrs',unit='deg')
        idx, d2d, d3d = c_measurements.match_to_catalog_sky(c_catalog,nthneighbor=1)
            
        MatchedTable=catalog[idx]
        stacked=hstack([MatchedTable,measurements])

  
        
        return stacked

    
    def Save_Catalog(self, catalog, path, coord=None, fov=None):
       
        catalog.meta['comments']=[str(coord.ra.degree),str(coord.dec.degree), str(fov)]

        name='catalog_'+str(round(coord.ra.degree,4))+'_'+str(round(coord.dec.degree,4))+'_'+str(round(fov,2))+'.cat'
        
        #name=catalog.meta['comments'][0]+'_'+catalog.meta['comments'][1]+'_'+catalog.meta['comments'][2]+'.cat'
        ascii.write(catalog,path+name,overwrite=True)
        
        return True
    

