#! python

# photometry pipeline

import time
start_time = time.time()

VERSION='0.11'

from optparse import OptionParser
import os, sys, shutil, warnings, logging

# do not disply all warnings
warnings.filterwarnings('ignore')

parser = OptionParser()
parser.add_option("-l", "--list", dest="list",
                  help="process all FITS files in LIST", metavar="LIST")
parser.add_option("-c", "--config", dest="configure",
                  help="configuration file CONF (default: phot.param)", metavar="CONF")  
(options, args) = parser.parse_args()
read_list=False

if len(args) != 1 :
	if (options.list):
		read_list = True
		list_file=options.list
		
	else:
		os.system('clear') 
		print() 
		print("                          photometry.py "+VERSION)
		print()
		print("photometry.py performe photometry cross-identification and calibration of all stars detected on CCD image")
		print()
		print("usage: python photometry.py directory [-l list_of_files][-c parameter_file]")
		print()
		print("       directory      - directory, where FITS files for photometry are stored ")
		print("       list_of_files  - list of FITS files for photometry")
		print("       parameter_file - file with parameters, that control flow of the program (defaut: photometry.param)")
		print()
		print("For more details and manual visit web page: http://astronomy.science.upjs.sk/research/sphotom/")
		sys.exit()		
else:
	if not sys.argv[1].endswith('/'):
		directory=sys.argv[1]+'/'
	else:
		directory=sys.argv[1]    
if not(options.configure):
	parameter_file="photometry.param"
else:
	parameter_file=options.configure	

import sqlite3
import numpy as np
from astropy.table import Table, Column, vstack
from astropy.coordinates.sky_coordinate import SkyCoord
from astropy import units as u
#from astroquery.xmatch import XMatch
from astropy.io import ascii
from astropy.io import fits
from astropy.coordinates import ICRS,SkyCoord
from astropy import units as u
import sep
import importlib

import modules.utils as utils
import modules.files as files
import modules.catalog as catalog
#import modules.astrometry as astrometry
import modules.calibration as calibration
import modules.database as database
import modules.phot as phot


# reading parameter file
try:
	param=utils.ParameterFile(parameter_file)
	global_parameters=param.Read()
	#print('OK')	
except:
	print('Cannot read parameter file!!')
	#print('Using default... not implemented yet, so skipping')
	sys.exit()	
    
os.system('clear')
print("                     photometry - pipeline v"+VERSION)
print('----------------------------------------------------------------------')
print()
if not(read_list):
    print( "Directory with FITS files: "+directory)
else:
    print( "List with FITS files: "+options.list)
    
print( "Parameter file: " + parameter_file)
print()
 
DEFAULT_PATH=global_parameters['DEFAULT_PATH']

if not DEFAULT_PATH.endswith('/'):
	DEFAULT_PATH=DEFAULT_PATH+'/'

TMP_DIR=DEFAULT_PATH+'tmp/'
#TMP_DIR=TMP_DIR_TOP+str(uuid.uuid4())+'/'
RESULTS_PATH=DEFAULT_PATH+'Results/'
PROCESSED_PATH=DEFAULT_PATH+'Processed/'	
CONFIG_PATH=DEFAULT_PATH+'Config/'	
COMP_PATH=DEFAULT_PATH+'Comparison/'
CATALOG_PATH=DEFAULT_PATH+'Catalog/'

if not(os.path.isdir(DEFAULT_PATH)):
    print('Creating default directory structure... ', end='')
    try:
        #os.makedirs(TMP_DIR_TOP)
        os.makedirs(PROCESSED_PATH)
        os.makedirs(RESULTS_PATH)
        os.makedirs(CONFIG_PATH)
        os.makedirs(COMP_PATH)
        os.makedirs(CATALOG_PATH)
        os.makedirs(TMP_DIR)
        sys.stdout.flush()   
        print('OK')     
    except:
        print('NO')
        print('Cannot create default directory structure! Check access permisions !!')
        print('Pipeline is terminated!')
        sys.exit()
        
else:   
    print('Checking default directory structure... ', end='')
    try:
        if not(os.path.isdir(PROCESSED_PATH)):
            os.makedirs(PROCESSED_PATH)
        if not(os.path.isdir(RESULTS_PATH)):
            os.makedirs(RESULTS_PATH)
        if not(os.path.isdir(CONFIG_PATH)):
            os.makedirs(CONFIG_PATH)
        if not(os.path.isdir(COMP_PATH)):
            os.makedirs(COMP_PATH)
        if not(os.path.isdir(CATALOG_PATH)):
            os.makedirs(CATALOG_PATH)
        if os.path.isdir(TMP_DIR):
            shutil.rmtree(TMP_DIR)
            os.makedirs(TMP_DIR)
        sys.stdout.flush()		
        print('OK')
    except:
        print('NO')
        print('Something is wrong with directory structure!!')
        print('Pipeline is terminated!')
        sys.exit()

PROGRESS=global_parameters['PROGRESS'].strip().upper()
PROGRESS_NAME=PROCESSED_PATH+'progress.db'

PHOTOMETRY = global_parameters['PHOTOMETRY'].strip().upper()
CALIBRATE = global_parameters['CALIBRATE'].strip().upper()
DATABASE=  global_parameters['DATABASE'].strip().upper()

if DATABASE=='Y':
	if global_parameters['DATABASE_NAME'].strip().upper()=='A':
		DATABASE_NAME=''
	else:
		DATABASE_NAME=RESULTS_PATH+ global_parameters['DATABASE_NAME'].strip()


#=====================================================================================
print()
print('Reading input images... ', end='')
#print()
sys.stdout.flush()	
if not(read_list):
	name_of_files=files.File_Names(directory)	
else:
	images = files.List(list=options.list)
	name_of_files=images.File_Names(lst=options.list)

print('OK')
print('Total number of images:',len(name_of_files))

first_image_parameters=files.Parameters(name_of_files[0])
dateobs=first_image_parameters['DATEOBS'][:10]

if PROGRESS=='Y':
    print()
    print('Reading progress database... ', end='')
    try:
        progress_connect, progress_cursor=utils.Progress.Connect(PROGRESS_NAME)
        print('OK')
    except:
        print('NO')
        print('Cannot connect to progress database. Continue without it!')
        PROGRESS='N'

    
DATABASE_NAME=RESULTS_PATH+'stars_'+dateobs+'.db'

FLAG=int(global_parameters['REMOVE_FLAG'].strip())
#initialize necessary modules

# old_obs data from progress database, where we store what was done
tmp_withwcs_files=[]
catalogs=[]
cat=catalog.Catalogs()
#astr=astrometry.Astrometry_NET()
calib=calibration.Calibration()
db=database.Database()
#diff=differential.Differential()
photometry=phot.Photometry_SEP(global_parameters, TMP_DIR)

stars_database, stars_database_cursor=db.Connect(DATABASE_NAME)
# ========================================================================================

for name in name_of_files:
    print('----------------------------------------------------------------------')
    print()
    if files.Compressed(name):
        #copy to tmp aand unpack
        pass
    else:
        image=files.Parameters(name)
        image['WORKING_NAME']=name
    if not image['IMAGE_TYPE']=='light':
        print('Processing image:',image['ORIGINAL_NAME'])
        print('Image is ',image['IMAGE_TYPE'].upper())
        print('Skipping from analysis')
    else:
        
        bar_cor=utils.Calculate_BJD(image)
        image['BJD']=image['JD']+bar_cor

        print('Processing image:',image['ORIGINAL_NAME'])
        print('BJD: %6.4f' %image['BJD'])
        print('Filter: ',image['FILTER'])
        print('Exposure time: %3.1f s' %(image['EXPTIME']))
        if image['WCS']=='Y':
            print('Object: ',image['OBJECT'])
            print('Central RA : %6.4f deg' %image['CRVAL1'])
            print('Central DEC: %6.4f deg' %image['CRVAL2'])
            print('     FOV X : %6.1f arcmin' %(image['FOV1']*60))
            print('     FOV Y : %6.1f arcmin' %(image['FOV2']*60))
            print('Pixel scale: %6.2f arcsec/px' %(image['PIXSCALE']*3600))
        else:
            print()
            print('Image has no WCS header, script cannot continue')
            break
        if PROGRESS=='Y':
            if utils.Progress.Check_In_Database(progress_connect,progress_cursor,image['JD']):
                what_to_do=utils.Progress.Check_What_To_Do(progress_connect,progress_cursor, image['JD'])
                    #print(what_to_do)
            
            else:
                #print('no')
                utils.Progress.Write_To_Database(progress_connect, progress_cursor, image)
                what_to_do={'test':False,'astrometry':True, 'sources':True, 'photometry':True, 'crossid':True, 'calibrate':True, 'database':True}
        else:        
            what_to_do={'test':False,'astrometry':True, 'sources':True, 'photometry':True, 'crossid':True, 'calibrate':True, 'database':True}
        
        print()
        if PHOTOMETRY=='Y':
            
            print('Estimating background... ', end='')  
            try:
                bkg=photometry.Background(image)
                print('OK')
                print('Background mean:',round(bkg.globalback,2))
                print('Background rms:', round(bkg.globalrms,2))
        #print()
            #save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'])
        
            #photometry.Background_Plot(bkg,save_name, normalize='sqrt', cmap='Greys')
                sys.stdout.flush()
            
            except:
                print('NO')
                print('Cannot determine background')
                continue
            if what_to_do['sources']:
                print()
                print('Finding possible sources: ', end='')                
                try:
                    sources=photometry.Sources(image,bkg)
                    print(len(sources))
                    save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='src')
                    ascii.write(sources,save_name)
                    #save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='src_plt')
                    #photometry.Sources_Plot(sources, data-bkg, save_name, normalize='log', cmap='Greys')
                    if PROGRESS=='Y':
                        utils.Progress.Update_Progress(progress_connect, progress_cursor, image['JD'], type='sources')
                except:
                    print('None')
                    print('Cannot find sources')
                    continue
            else:
                # read sources from file previously processed
                print()
                print('Reading saved sources... ',end='')
                try:
                    save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='src')
                    sources=ascii.read(save_name)
                    print('OK')
                    print('Number of possible sources: ',len(sources))
                except:
                    print('NO')
                    print('Cannot find stored sources')
                    continue
            if what_to_do['photometry']:
                print()
                sys.stdout.flush()
                print('Performing aperture photometry... ', end='')
                try:
                    sys.stdout.flush()
                    tmp=photometry.Photometry(sources, image, bkg, aper_type='mag_auto')
                    #print(tmp)
                    tmp['ALPHA'], tmp['DELTA']=utils.Convert_To_WORLD(image['WORKING_NAME'],tmp)
                    measurements=utils.Clear(tmp,flag=FLAG)
                    sys.stdout.flush()
                    print('OK')
                    print('Number of sources for analysis...',len(measurements))
                    save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='phot')
                    ascii.write(measurements,save_name)
                    if PROGRESS=='Y':
                        utils.Progress.Update_Progress(progress_connect, progress_cursor, image['JD'], type='photometry')
    
                except:
                    print('NO')
                    print('Cannot make photometry')
                    continue
                    
            else:
                print()
                print('Reading saved photometry... ',end='')
                try:
                    save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='phot')
                    measurements=ascii.read(save_name)
                    print('OK')
                    print('Number of sources for analysis...',len(measurements))
                    #print(measurements)
                except:
                    print('NO')
                    print('Cannot find storedphotometry')
                    continue
        else:
            print('NO photometry')
            
        if CALIBRATE=='Y':
            if image['WCS']=='Y':
                if what_to_do['crossid']:
                    print()
                    #print('Crossidentification... ')
                    print('Loading catalog...', end='')
                    if image['WCS']=='Y':
                        ra=image['CENTER1']
                        dec=image['CENTER2']
                        star=image['OBJECT']	
                        fov=max(image['FOV1'], image['FOV2'])*1.2
                        c=SkyCoord(ra, dec ,unit=(u.deg, u.deg), frame='icrs')
                        cat_path=CATALOG_PATH+dateobs+'/'
                        sys.stdout.flush()
                        if not os.path.isdir(cat_path):
                            os.makedirs(cat_path)
                        catalog,cat_name = cat.Find_Local_Catalog(c,cat_path)
                        if len(catalog)==0:  
                            sys.stdout.flush()
                            catalog_USNOB=cat.Load_Vizier(c,fov=fov, catalog='USNOB')
                            catalog_VSX=cat.Load_Vizier(c,fov=fov, catalog='VSX')
                            catalog_APASS=cat.Load_Vizier(c,fov=fov, catalog='APASS')                
                            catalog=cat.Match_All(catalog_USNOB, catalog_APASS, catalog_VSX)
                            cat.Save_Catalog(catalog,cat_path, coord=c, fov=fov)
                        print('OK')
                        print('Starting crossidentification... ', end='')
                        sys.stdout.flush()
                        matched_catalog=cat.Match(measurements, catalog)
                        save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='crossid')
                        ascii.write(matched_catalog,save_name)
                        if PROGRESS=='Y':
                            utils.Progress.Update_Progress(progress_connect, progress_cursor, image['JD'], type='crossid')
                        print('OK')
                else:
                    print()
                    print('Reading saved crossidentification... ',end='')
                    try:
                        save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='crossid')
                        matched_catalog=ascii.read(save_name)
                        print('OK')
                        #print(measurements)
                    except:
                        print('NO')
                if what_to_do['calibrate']:
                    sys.stdout.flush()
                    print()
                    print('Photometric calibration... ')
                    calibrated_catalog, zero_point, zero_point_error, number =calib.Calibrate(matched_catalog,filter=image['FILTER'])
                    print('Zero point: {:4.4f} +- {:4.4f}'.format(zero_point,zero_point_error))
                    print('Number of calibration stars: ',number)
                    calibrated_catalog=utils.New_Commnets(calibrated_catalog, zero_point, zero_point_error, number)
                    #print(calibrated_catalog.meta)
                    
                    save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='calib')
                    ascii.write(calibrated_catalog,save_name)
                    if PROGRESS=='Y':
                        utils.Progress.Update_Progress(progress_connect, progress_cursor, image['JD'], type='calib')
                    
                else:
                    print()
                    print('Reading saved calibration... ',end='')
                    try:
                        save_name=utils.Save_Name(PROCESSED_PATH+dateobs, image['WORKING_NAME'], type='calib')
                        calibrated_catalog=ascii.read(save_name)
                        print('OK')
                        #print(measurements)
                    except:
                        print('NO') 
                #saving to database
                print()
                save_name=utils.Save_Name(RESULTS_PATH+dateobs, image['TMP_NAME'], type='fits')
                files.Save_To_FITS(save_name, calibrated_catalog, image)
                if DATABASE=='Y':
                    print('Saving to database... ', end='')
                    sys.stdout.flush()
                    
                    if PROGRESS=='Y':
                        rewrite=False
                    else:
                        rewrite=True
                    sys.stdout.flush()
                    db.Save_LC(stars_database, stars_database_cursor, calibrated_catalog, image, rewrite=rewrite)
                    print('OK')
                    if PROGRESS=='Y':
                        utils.Progress.Update_Progress(progress_connect, progress_cursor, image['JD'], type='database')
                   

                
                
            else:
                print('NO')
                print('Image have no astrometry solution so no calibration can be performed')
                print('Skipping file from other analysis')
                        #do all staf
                   
        else:
            print('No calibration is selected')
            print('Skipping file from other analysis')
            
        #savin to database
                    
        
print()
execute_time=time.time() - start_time
m, s = divmod(execute_time, 60)
h, m = divmod(m, 60)	
out='This script was executed %dh %02dm %02d seconds' %(h, m, s)
print()
print(out)
sys.exit()
                         

